<?php

class MessageController extends Controller
{
	public $defaultAction = 'inbox';
	public $buttons = array( // Ex output: {count} messages have been {value}
				'default'=>array('delete'=>'deleted','read'=>'marked read','unread'=>'marked unread'),
				'trash'=>array('delete'=>'permanently deleted','restore'=>'restored')
		);
	
	public function filters()
	{
		if($this->module->authManager=='rights') {
			return array(
				'rights', // perform access control for CRUD operations
			);
		}
		else return array();
	}
	
	public function actionInbox()
	{
		if(isset($_POST['convs']))
		{
			$this->buttonAction('inbox');
		}
		$dataProvider = new CActiveDataProvider(Mailbox::model()->inbox());
		$this->render('mailbox',array('dataProvider'=>$dataProvider));
	}
	
	public function actionSent()
	{
		if(!$this->module->authManager && (!$this->module->sentbox  || ($this->module->readOnly && !MailboxModule::isAdmin()) ) )
			$this->redirect(array('message/inbox'));
			
		if(isset($_POST['convs']))
		{
			$this->buttonAction('sent');
		}
		$dataProvider = new CActiveDataProvider(Message::model()->sent());
		//echo "<pre>";
		//CVarDumper::dump(Message::model()->sent()->getCommandBuilder()->createFindCommand(Message::$tableName,Message::model()->sent()->getDbCriteria())->getText());
		//exit;
		$this->render('mailbox',array('dataProvider'=>$dataProvider));
	}
	
	
	public function actionTrash()
	{
		if(!$this->module->authManager && (!MailboxModule::$trashbox  || ($this->module->readOnly && !MailboxModule::isAdmin()) ))
			$this->redirect(array('message/inbox'));
		if(isset($_POST['convs']))
		{
			$this->buttonAction('trash','trash');
		}
		$period =& MailboxModule::$recyclePeriod;
		Yii::app()->user->setFlash('notice', "Messages in the trash are deleted within {$period} days.");
		$dataProvider = new CActiveDataProvider(Mailbox::model()->trash());
		$this->render('mailbox',array('dataProvider'=>$dataProvider));
	}
	
	public function actionNew()
	{
		if(!$this->module->authManager && (!$this->module->sendMsgs  || ($this->module->readOnly && !MailboxModule::isAdmin()) ))
			$this->redirect(array('message/inbox'));
		
		if(isset($_POST['Mailbox']['to']))
		{
			$t = time();
			$conv = new Mailbox();
			$conv->subject = ($_POST['Mailbox']['subject'])? $_POST['Mailbox']['subject'] : $this->module->defaultSubject;
			$conv->to = $_POST['Mailbox']['to'];
			$conv->initiator_id = MailboxModule::getUserId();

			// Check if username exist
			if(strlen($_POST['Mailbox']['to'])>1)
				$conv->interlocutor_id = MailboxModule::getUserId($_POST['Mailbox']['to']);
			else
				$conv->interlocutor_id = 0;
			// ...if not check if To field is user id
			if(!$conv->interlocutor_id)
			{
				if($_POST['Mailbox']['to'] && $this->module->allowLookupById)
					$username = MailboxModule::getUserName($_POST['Mailbox']['to']);
				if(@$username)
				{
					$conv->interlocutor_id = $_POST['Mailbox']['to'];
					$conv->to = $username;
				}
				else
				{
					$conv->addError('to','User not found?');
				}
			}
			
			if($conv->interlocutor_id && $conv->initiator_id == $conv->interlocutor_id) {
				$conv->addError('to', "Can't send message to self!");
			}
			
			// check user-to-user perms
			if(!$conv->hasErrors() && !$this->module->userToUser && !MailboxModule::isAdmin())
			{
				if(!MailboxModule::isAdmin($conv->to))
					$conv->addError('to', "Invalid user!");
			}
			
			$conv->modified = $t;
			$conv->bm_read = Mailbox::INITIATOR_FLAG;
			$msg = new Message();
			$msg->text = $_POST['Message']['text'];
			$validate = $conv->validate(array('text'),false); // html purify
			$msg->created = $t;
			$msg->sender_id = $conv->initiator_id;
			$msg->recipient_id = $conv->interlocutor_id;
			if($this->module->checksums) {
				$msg->crc64 = Message::crc64($msg->text); // 64bit INT
			}
			else
				$msg->crc64 = 0;
			// Validate
			$validate = $conv->validate(null,false); // don't clear errors
			$validate = $msg->validate() && $validate;
			
			if($validate)
			{
				$conv->save();
				$msg->conversation_id = $conv->conversation_id;
				$msg->save();
				Yii::app()->user->setFlash('success', "Message has been sent!");
				$this->redirect(array('message/inbox'));
			}
			else
			{
				Yii::app()->user->setFlash('error', "Error sending message!");
			}
		}
		else{
			$conv = new Mailbox();
			if(isset($_GET['id']))
				$conv->to = MailboxModule::getUserName($_GET['id']);
			elseif(isset($_GET['to']))
				$conv->to = $_GET['to'];
			else
				$conv->to = '';
			$msg = new Message();
		}
		$this->render('compose',array('conv'=>$conv,'msg'=>$msg));
	}
	
	public function actionReply()
	{
		if(!$this->module->authManager && (!$this->module->sendMsgs  || ($this->module->readOnly && !MailboxModule::isAdmin()) ))
			$this->redirect(array('message/inbox'));
		$reply = new Message;
		$conv = Mailbox::conversation($_GET['id']);
		
		if(isset($_POST['text']))
		{
			$reply->text = $_POST['text'];
			$validate = $conv->validate(array('text'),false); // html purify
			$reply->conversation_id = $conv->conversation_id;
			if($conv->initiator_id != Yii::app()->user->id) {
				$reply->recipient_id = $conv->initiator_id;
				$conv->bm_read = $conv->bm_read & ~Mailbox::INITIATOR_FLAG;
			}
			else {
				$reply->recipient_id = $conv->interlocutor_id;
				$conv->bm_read = $conv->bm_read & ~Mailbox::INTERLOCUTOR_FLAG;
			}
			
			$reply->sender_id = MailboxModule::getUserId();

			$reply->created = time();
			$conv->modified = $reply->created;
			
			$reply->crc64 = Message::crc64($reply->text);
			
			$conv->bm_deleted = 0; // restore message
			$conv->interlocutor_del = 0;
			$conv->initiator_del = 0;
			
			$validate = $reply->validate();
			$validate = $conv->validate() && $validate;
			
			if($validate)
			{
				$conv->save();
				$reply->save();
				Yii::app()->user->setFlash('success', "Message sent!");
				$this->redirect(array('message/inbox'));
			}
			else{
				Yii::app()->user->setFlash('error', "Error sending message!");
			
				$this->render('message',array('conv'=>$conv, 'reply'=>$reply));
			}
		}
	}
	
	public function actionView()
	{
		$conv = Mailbox::conversation($_GET['id']);
		
		$conv->markRead();
		$reply = new Message;
		$this->render('message',array('conv'=>$conv, 'reply'=>$reply));
		
	}
	
	public function buttonAction($box='inbox' , $buttonset='default')
	{
		if(!$_POST['button']) {
			if($_GET['ajax'])
				die('{"error":"Action not found?"}');
			Yii::app()->user->setFlash('error', "Action not found?");
			$this->redirect(array('message/'.$box));
		}
		$action = key($_POST['button']);
		if(!array_key_exists($action,$this->buttons[$buttonset]))
			throw new Exception('Button action not found?');
		
		$partialmsg = $this->buttons[$buttonset][$action];
		
		$count=0;
		foreach($_POST['convs'] as &$conversation_id)
		{
			/*
			* None of the following errors should happen unless the user 
			* tampers with the input vars, so we ignore them and continue
			*/
			if(!is_int((int)$conversation_id))
				continue;

			$conv = Mailbox::model()->findByPk($conversation_id);

			if(!$conv->belongsTo(MailboxModule::getUserId()))
				continue;
			if(!$conv->$action(MailboxModule::getUserId()) || !$conv->validate())
				continue;
			if($conv->save())
				$count++;
		}
		
		if($count)
		{
			$message = $count." message(s) have been {$partialmsg}!";
			if($_GET['ajax'])
			{
				if($_GET['ajaxdelete'])
					die('{"success":"+'.$count.' deleted!"}');
				die('{"success":"'.$message.'"}');
			}
			Yii::app()->user->setFlash('success', $message);
			$this->redirect(array('message/'.$box));
		}
		else
		{
			$message = "Message could not be {$partialmsg}!";
			if($_GET['ajax'])
			{
				if($_GET['ajaxdelete'])
					die('{"error":"Error deleting?"}');
				die('{"error":"'.$message.'"}');
			}
			Yii::app()->user->setFlash('error', $message);
			$this->redirect(array('message/'.$box));
		}
	}
	
}
